module Main where
import GHC.Conc(par)
import Control.Parallel.Strategies hiding(parMap)
import Control.DeepSeq
import Data.List.Split
import System.Environment (getArgs)
import System.Exit (exitSuccess)
import Grid (getSolvedGrid, Grid)
import Solver (solve, solve')
import Logger
import Parser

checkArgs :: [String] -> IO [String]
checkArgs xs = if null xs then displayHelp >> exitSuccess else pure xs

inputToGrid :: String -> [[Int]]
inputToGrid input = transformToGrid (words <$> (drop 1 . clearInput . lines $ input))

solveNPuzzle :: Grid -> Int
solveNPuzzle grid = solve' (getSolvedGrid $ length grid) grid (Nothing, Nothing)

parMap :: (a -> b) -> [a] -> Eval [b]
parMap _ [] = return []
parMap f (a:as) = do b <- rpar (f a)
                     bs <- parMap f as
                     return (b:bs)

main :: IO ()
main = do
    args <- checkArgs =<< getArgs
    content <- readFile $ args !! 0
    let input_list = splitOn "#" content
        grids = map inputToGrid input_list
        (as, bs) = splitAt (length grids `div` 2) grids
        solution = runEval $ do
                    as' <- rpar (force (map solveNPuzzle as))
                    bs' <- rpar (force (map solveNPuzzle bs))
                    rseq as'
                    rseq bs'
                    return (as'++ bs')
    print (length solution) >> mapM_ print solution

    {- map solveNPuzzle grids -}
    {- runEval (parMap solveNPuzzle grids) -}
    {- runEval $ do
                as' <- rpar (force (map solveNPuzzle as))
                bs' <- rpar (force (map solveNPuzzle bs))
                rseq as'
                rseq bs'
                return (as'++ bs') -}
